Package: GloScope
Type: Package
Title: Population-level Representation on scRNA-Seq data
Version: 2.3.0
Authors@R: c(
  person("Elizabeth", "Purdom", email = "epurdom@stat.berkeley.edu", role = c("aut","cre")),
  person("William", "Torous", email = "wtorous@berkeley.edu",
            role = c("aut"),
	    comment = c(ORCID="0000-0001-5668-5510")),
  person("Hao", "Wang", email = "hao_wang@berkeley.edu",
            role = c("aut"),
            comment = c(ORCID="0000-0002-0749-474X")),
  person("Boying", "Gong", email = "boyinggong@berkeley.edu", role = "aut"))
Description: This package aims at representing and summarizing the entire single-cell profile of 
  a sample. It allows researchers to perform important bioinformatic analyses at the sample-level 
  such as visualization and quality control. The main functions Estimate sample distribution and calculate
  statistical divergence among samples, and visualize the distance matrix through MDS plots. 
BugReports: https://github.com/epurdom/GloScope/issues
License: Artistic-2.0
Encoding: UTF-8
Imports: utils, stats, MASS, mclust, ggplot2, RANN, FNN, BiocParallel,
        mvnfast, SingleCellExperiment, rlang, RColorBrewer, pheatmap,
        vegan, cluster, boot, permute
Depends: R (>= 4.4.0)
Suggests: BiocStyle, testthat (>= 3.0.0), knitr, rmarkdown,
        zellkonverter
VignetteBuilder: knitr
LazyData: false
biocViews: DataRepresentation, QualityControl, RNASeq, Sequencing,
        Software, SingleCell
RoxygenNote: 7.3.3
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/GloScope
git_branch: devel
git_last_commit: 15d93b8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:54:52 UTC; biocbuild
Author: Elizabeth Purdom [aut, cre],
  William Torous [aut] (ORCID: <https://orcid.org/0000-0001-5668-5510>),
  Hao Wang [aut] (ORCID: <https://orcid.org/0000-0002-0749-474X>),
  Boying Gong [aut]
Maintainer: Elizabeth Purdom <epurdom@stat.berkeley.edu>
