# Generated by roxygen2: do not edit by hand

export(bootCI)
export(bootGloscope)
export(getMetrics)
export(gloscope)
export(gloscopeProp)
export(plotCI)
export(plotHeatmap)
export(plotMDS)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(FNN,KL.dist)
importFrom(MASS,isoMDS)
importFrom(RANN,nn2)
importFrom(RColorBrewer,brewer.pal)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,colData)
importFrom(SingleCellExperiment,reducedDim)
importFrom(boot,boot)
importFrom(boot,boot.ci)
importFrom(cluster,silhouette)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_pointrange)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,position_dodge)
importFrom(grDevices,colorRampPalette)
importFrom(mclust,densityMclust)
importFrom(mvnfast,rmvn)
importFrom(permute,how)
importFrom(permute,permute)
importFrom(pheatmap,pheatmap)
importFrom(rlang,.data)
importFrom(stats,as.dist)
importFrom(stats,predict)
importFrom(stats,rmultinom)
importFrom(utils,combn)
importFrom(vegan,adonis2)
importFrom(vegan,anosim)
